@@ -49,7 +49,7 @@ struct DownloadHelper
 
   gchar *referer;
   gchar *user_agent;
-  gchar **cookies;
+  GSList *cookies;
 };
 
 struct DownloadHelperTransfer
@@ -64,7 +64,14 @@ struct DownloadHelperTransfer
 
   GCancellable *cancellable;
 
-  SoupMessage *msg;
+  gboolean is_file_transfer;
+
+  union
+  {
+    SoupMessage *msg;
+    GFile *file;
+  };
+
   gboolean request_sent;
 
   /* Current read buffer */
@@ -99,7 +106,7 @@ transfer_completion_cb (gpointer src_object, GAsyncResult * res,
   DownloadRequest *request = transfer->request;
 
   if (transfer->blocking)
-    return;                     /* Somehow a completion got signalled for a blocking request */
+    return;
 
   download_request_lock (request);
   request->in_use = FALSE;
@@ -134,11 +141,10 @@ transfer_report_progress_cb (gpointer task)
   return FALSE;
 }
 
-static GTask *
-transfer_task_new (DownloadHelper * dh, DownloadRequest * request,
-    SoupMessage * msg, gboolean blocking)
+static DownloadHelperTransfer *
+transfer_new_common (DownloadHelper * dh, DownloadRequest * request,
+    gboolean blocking)
 {
-  GTask *transfer_task = NULL;
   DownloadHelperTransfer *transfer = g_new0 (DownloadHelperTransfer, 1);
 
   transfer->blocking = blocking;
@@ -149,8 +155,13 @@ transfer_task_new (DownloadHelper * dh, DownloadRequest * request,
   transfer->request = download_request_ref (request);
 
   transfer->dh = dh;
-  transfer->msg = msg;
+  return transfer;
+}
 
+static GTask *
+transfer_task_new_for_transfer (DownloadHelperTransfer * transfer)
+{
+  GTask *transfer_task = NULL;
   transfer_task =
       g_task_new (NULL, transfer->cancellable,
       (GAsyncReadyCallback) transfer_completion_cb, NULL);
@@ -160,6 +171,28 @@ transfer_task_new (DownloadHelper * dh, DownloadRequest * request,
   return transfer_task;
 }
 
+static GTask *
+transfer_task_new_file (DownloadHelper * dh, DownloadRequest * request,
+    gboolean blocking)
+{
+  DownloadHelperTransfer *transfer =
+      transfer_new_common (dh, request, blocking);
+  transfer->is_file_transfer = TRUE;
+  transfer->file = g_file_new_for_uri (request->uri);
+
+  return transfer_task_new_for_transfer (transfer);
+}
+
+static GTask *
+transfer_task_new_soup (DownloadHelper * dh, DownloadRequest * request,
+    SoupMessage * msg, gboolean blocking)
+{
+  DownloadHelperTransfer *transfer =
+      transfer_new_common (dh, request, blocking);
+  transfer->msg = msg;
+  return transfer_task_new_for_transfer (transfer);
+}
+
 static void
 release_transfer_task_by_ref (GTask ** transfer_task)
 {
@@ -201,7 +234,8 @@ finish_transfer_task (DownloadHelper * dh, GTask * transfer_task,
 
       if (transfer->blocking)
         g_cond_broadcast (&transfer->cond);
-      else if (error != NULL)
+
+      if (error != NULL)
         g_task_return_error (transfer_task, error);
       else
         g_task_return_boolean (transfer_task, TRUE);
@@ -222,19 +256,21 @@ static gboolean
 new_read_buffer (DownloadHelperTransfer * transfer)
 {
   gint buffer_size = CHUNK_BUFFER_SIZE;
-#if 0
-  DownloadRequest *request = transfer->request;
 
-  if (request->range_end != -1) {
-    if (request->range_end <= transfer->read_position) {
-      transfer->read_buffer = NULL;
-      transfer->read_buffer_size = 0;
-      return FALSE;
+  if (transfer->is_file_transfer) {
+    /* For file reads, handle range request limits here */
+    DownloadRequest *request = transfer->request;
+
+    if (request->range_end != -1) {
+      if (request->range_end <= transfer->read_position) {
+        transfer->read_buffer = NULL;
+        transfer->read_buffer_size = 0;
+        return FALSE;
+      }
+      if (request->range_end - transfer->read_position < buffer_size)
+        buffer_size = request->range_end - transfer->read_position + 1;
     }
-    if (request->range_end - transfer->read_position < buffer_size)
-      buffer_size = request->range_end - transfer->read_position + 1;
   }
-#endif
 
   transfer->read_buffer = g_new (char, buffer_size);
   transfer->read_buffer_size = buffer_size;
@@ -267,7 +303,7 @@ on_read_ready (GObject * source, GAsyncResult * result, gpointer user_data)
 
     if (!g_cancellable_is_cancelled (transfer->cancellable)) {
       GST_ERROR ("Failed to read stream: %s", error->message);
-      if (request->state != DOWNLOAD_REQUEST_STATE_UNSENT)
+      if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED)
         request->state = DOWNLOAD_REQUEST_STATE_ERROR;
       finish_transfer_task (dh, transfer_task, error);
     } else {
@@ -309,19 +345,19 @@ on_read_ready (GObject * source, GAsyncResult * result, gpointer user_data)
       }
     }
 
-    if (request->download_start_time == GST_CLOCK_TIME_NONE) {
-      GST_LOG ("Got first data for URI %s", request->uri);
-      request->download_start_time = now;
-    }
-
     if (gst_buffer != NULL) {
-      /* Unsent means cancellation is in progress, so don't override
-       * the state. Otherwise make sure it is LOADING */
-      if (request->state != DOWNLOAD_REQUEST_STATE_UNSENT)
+      /* Don't override CANCELLED state. Otherwise make sure it is LOADING */
+      if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED)
         request->state = DOWNLOAD_REQUEST_STATE_LOADING;
 
-      GST_LOG ("Adding %u bytes to buffer",
-          (guint) (gst_buffer_get_size (gst_buffer)));
+      if (request->download_start_time == GST_CLOCK_TIME_NONE) {
+        GST_LOG ("Got first data for URI %s", request->uri);
+        request->download_start_time = now;
+      }
+      request->download_newest_data_time = now;
+
+      GST_LOG ("Adding %u bytes to buffer (request URI %s)",
+          (guint) (gst_buffer_get_size (gst_buffer)), request->uri);
 
       download_request_add_buffer (request, gst_buffer);
 
@@ -347,18 +383,23 @@ on_read_ready (GObject * source, GAsyncResult * result, gpointer user_data)
 
 finish_transfer:
   if (request->in_use && !g_cancellable_is_cancelled (transfer->cancellable)) {
-    SoupStatus status_code = _soup_message_get_status (transfer->msg);
+    if (!transfer->is_file_transfer) {
+      request->status_code = _soup_message_get_status (transfer->msg);
+    }
+#ifndef GST_DISABLE_GST_DEBUG
+    guint download_ms = (now - request->download_request_time) / GST_MSECOND;
+    GST_LOG ("request complete in %u ms. Code %d URI %s range %" G_GINT64_FORMAT
+        " %" G_GINT64_FORMAT, download_ms, request->status_code,
+        request->uri, request->range_start, request->range_end);
+#endif
 
-    GST_LOG ("request complete. Code %d URI %s range %" G_GINT64_FORMAT " %"
-        G_GINT64_FORMAT, status_code, request->uri,
-        request->range_start, request->range_end);
-
-    if (request->state != DOWNLOAD_REQUEST_STATE_UNSENT) {
-      if (SOUP_STATUS_IS_SUCCESSFUL (status_code)
-          || SOUP_STATUS_IS_REDIRECTION (status_code))
+    if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED) {
+      if (SOUP_STATUS_IS_SUCCESSFUL (request->status_code)
+          || SOUP_STATUS_IS_REDIRECTION (request->status_code)) {
         request->state = DOWNLOAD_REQUEST_STATE_COMPLETE;
-      else
+      } else {
         request->state = DOWNLOAD_REQUEST_STATE_ERROR;
+      }
     }
   }
   request->download_end_time = now;
@@ -504,6 +545,115 @@ handle_response_headers (DownloadHelperTransfer * transfer)
   return http_headers;
 }
 
+static void
+on_file_ready (GObject * source, GAsyncResult * result, gpointer user_data)
+{
+  GTask *transfer_task = user_data;
+  DownloadHelperTransfer *transfer = g_task_get_task_data (transfer_task);
+
+  DownloadHelper *dh = transfer->dh;
+  DownloadRequest *request = transfer->request;
+  GError *error = NULL;
+
+  GFileInputStream *in = g_file_read_finish (transfer->file, result, &error);
+  download_request_lock (request);
+
+  if (in == NULL) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
+      request->status_code = SOUP_STATUS_NOT_FOUND;
+    } else if (g_error_matches (error, G_IO_ERROR,
+            G_IO_ERROR_PERMISSION_DENIED)) {
+      request->status_code = SOUP_STATUS_FORBIDDEN;
+    } else {
+      request->status_code = SOUP_STATUS_INTERNAL_SERVER_ERROR;
+    }
+
+    if (!g_cancellable_is_cancelled (transfer->cancellable)) {
+      GST_LOG ("request errored. Code %d URI %s range %" G_GINT64_FORMAT " %"
+          G_GINT64_FORMAT, request->status_code, request->uri,
+          request->range_start, request->range_end);
+
+      if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED)
+        request->state = DOWNLOAD_REQUEST_STATE_ERROR;
+      finish_transfer_task (dh, transfer_task, error);
+    } else {
+      /* Ignore error from cancelled operation */
+      g_error_free (error);
+      finish_transfer_task (dh, transfer_task, NULL);
+    }
+    download_request_unlock (request);
+
+    /* No async callback queued - the transfer is done */
+    finish_transfer_task (dh, transfer_task, error);
+    return;
+  }
+
+  /* If the state is cancelled don't override it */
+  if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED &&
+      request->state != DOWNLOAD_REQUEST_STATE_HEADERS_RECEIVED) {
+
+    request->state = DOWNLOAD_REQUEST_STATE_HEADERS_RECEIVED;
+    request->status_code = SOUP_STATUS_OK;
+    // FIXME: request->headers = handle_response_headers (transfer);
+
+    GST_TRACE ("request URI %s range %" G_GINT64_FORMAT " %"
+        G_GINT64_FORMAT " headers: %" GST_PTR_FORMAT,
+        request->uri, request->range_start, request->range_end,
+        request->headers);
+
+    if (SOUP_STATUS_IS_SUCCESSFUL (request->status_code)
+        || SOUP_STATUS_IS_REDIRECTION (request->status_code)) {
+      request->state = DOWNLOAD_REQUEST_STATE_HEADERS_RECEIVED;
+      transfer_task_report_progress (transfer_task);
+    } else {
+      goto finish_transfer_error;
+    }
+  }
+
+  if (!new_read_buffer (transfer))
+    goto finish_transfer_error;
+
+  /* Respect any range request */
+  if (request->range_start != 0) {
+    if (!g_seekable_seek (G_SEEKABLE (in), request->range_start, G_SEEK_SET,
+            transfer->cancellable, &error)) {
+      goto finish_transfer_error;
+    }
+    transfer->read_position = request->range_start;
+  }
+
+  download_request_unlock (request);
+
+  g_main_context_push_thread_default (dh->transfer_context);
+  g_input_stream_read_all_async (G_INPUT_STREAM (in), transfer->read_buffer,
+      transfer->read_buffer_size, G_PRIORITY_DEFAULT, transfer->cancellable,
+      on_read_ready, transfer_task);
+  g_main_context_pop_thread_default (dh->transfer_context);
+
+  g_object_unref (in);
+  return;
+
+finish_transfer_error:
+  request->download_end_time = gst_adaptive_demux_clock_get_time (dh->clock);
+
+  if (request->in_use && !g_cancellable_is_cancelled (transfer->cancellable)) {
+    GST_LOG ("request complete. Code %d URI %s range %" G_GINT64_FORMAT " %"
+        G_GINT64_FORMAT, request->status_code, request->uri,
+        request->range_start, request->range_end);
+
+    /* If the state is cancelled don't override it */
+    if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED)
+      request->state = DOWNLOAD_REQUEST_STATE_ERROR;
+  }
+
+  g_free (transfer->read_buffer);
+  transfer->read_buffer = NULL;
+
+  download_request_unlock (request);
+  finish_transfer_task (dh, transfer_task, NULL);
+  g_object_unref (in);
+}
+
 static void
 on_request_sent (GObject * source, GAsyncResult * result, gpointer user_data)
 {
@@ -528,7 +678,7 @@ on_request_sent (GObject * source, GAsyncResult * result, gpointer user_data)
           G_GINT64_FORMAT, request->status_code, request->uri,
           request->range_start, request->range_end);
 
-      if (request->state != DOWNLOAD_REQUEST_STATE_UNSENT)
+      if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED)
         request->state = DOWNLOAD_REQUEST_STATE_ERROR;
       finish_transfer_task (dh, transfer_task, error);
     } else {
@@ -543,8 +693,8 @@ on_request_sent (GObject * source, GAsyncResult * result, gpointer user_data)
     return;
   }
 
-  /* If the state went back to UNSENT, we were cancelled so don't override it */
-  if (request->state != DOWNLOAD_REQUEST_STATE_UNSENT &&
+  /* If the state is cancelled don't override it */
+  if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED &&
       request->state != DOWNLOAD_REQUEST_STATE_HEADERS_RECEIVED) {
 
     request->state = DOWNLOAD_REQUEST_STATE_HEADERS_RECEIVED;
@@ -585,7 +735,9 @@ finish_transfer_error:
     GST_LOG ("request complete. Code %d URI %s range %" G_GINT64_FORMAT " %"
         G_GINT64_FORMAT, _soup_message_get_status (msg), request->uri,
         request->range_start, request->range_end);
-    if (request->state != DOWNLOAD_REQUEST_STATE_UNSENT)
+
+    /* If the state is cancelled don't override it */
+    if (request->state != DOWNLOAD_REQUEST_STATE_CANCELLED)
       request->state = DOWNLOAD_REQUEST_STATE_ERROR;
   }
 
@@ -597,6 +749,44 @@ finish_transfer_error:
   g_object_unref (in);
 }
 
+static inline gchar
+gst_soup_util_log_make_level_tag (SoupLoggerLogLevel level)
+{
+  gchar c;
+
+  if (G_UNLIKELY ((gint) level > 9))
+    return '?';
+
+  switch (level) {
+    case SOUP_LOGGER_LOG_MINIMAL:
+      c = 'M';
+      break;
+    case SOUP_LOGGER_LOG_HEADERS:
+      c = 'H';
+      break;
+    case SOUP_LOGGER_LOG_BODY:
+      c = 'B';
+      break;
+    default:
+      /* Unknown level. If this is hit libsoup likely added a new
+       * log level to SoupLoggerLogLevel and it should be added
+       * as a case */
+      c = level + '0';
+      break;
+  }
+  return c;
+}
+
+static void
+gst_soup_util_log_printer_cb (SoupLogger G_GNUC_UNUSED * logger,
+    SoupLoggerLogLevel level, char direction, const char *data,
+    gpointer user_data)
+{
+  gchar c;
+  c = gst_soup_util_log_make_level_tag (level);
+  GST_TRACE ("HTTP_SESSION(%c): %c %s", c, direction, data);
+}
+
 DownloadHelper *
 downloadhelper_new (GstAdaptiveDemuxClock * clock)
 {
@@ -625,6 +815,17 @@ downloadhelper_new (GstAdaptiveDemuxClock * clock)
    * an attempt to reuse an already closed connection */
   dh->session = _soup_session_new_with_options ("timeout", 10, NULL);
 
+  /* Setup soup header debugging if we are at GST_LEVEL_TRACE */
+  if (gst_debug_category_get_threshold (GST_CAT_DEFAULT) >= GST_LEVEL_TRACE) {
+    /* Create a new logger and set body_size_limit to -1 (no limit) */
+    SoupLogger *logger = _soup_logger_new (SOUP_LOGGER_LOG_HEADERS);
+
+    _soup_logger_set_printer (logger, gst_soup_util_log_printer_cb, NULL, NULL);
+
+    /* Attach logger to session */
+    _soup_session_add_feature (dh->session, (SoupSessionFeature *) logger);
+    g_object_unref (logger);
+  }
   g_main_context_pop_thread_default (dh->transfer_context);
 
   return dh;
@@ -648,7 +849,7 @@ downloadhelper_free (DownloadHelper * dh)
 
   g_free (dh->referer);
   g_free (dh->user_agent);
-  g_strfreev (dh->cookies);
+  _soup_cookies_free (dh->cookies);
 
   g_free (dh);
 }
@@ -675,10 +876,25 @@ downloadhelper_set_user_agent (DownloadHelper * dh, const gchar * user_agent)
 void
 downloadhelper_set_cookies (DownloadHelper * dh, gchar ** cookies)
 {
+  guint i;
   g_mutex_lock (&dh->transfer_lock);
-  g_strfreev (dh->cookies);
-  dh->cookies = cookies;
+  _soup_cookies_free (dh->cookies);
+  dh->cookies = NULL;
+
+  for (i = 0; cookies[i]; i++) {
+    SoupCookie *cookie = _soup_cookie_parse (cookies[i]);
+
+    if (cookie == NULL) {
+      GST_WARNING ("Couldn't parse cookie, ignoring: %s", cookies[i]);
+      continue;
+    }
+
+    dh->cookies = g_slist_append (dh->cookies, cookie);
+  }
+
   g_mutex_unlock (&dh->transfer_lock);
+
+  g_strfreev (cookies);
 }
 
 /* Called with the transfer lock held */
@@ -689,6 +905,13 @@ submit_transfer (DownloadHelper * dh, GTask * transfer_task)
   DownloadRequest *request = transfer->request;
 
   download_request_lock (request);
+  if (request->state == DOWNLOAD_REQUEST_STATE_CANCELLED) {
+    download_request_unlock (request);
+
+    GST_DEBUG ("Don't submit already cancelled transfer");
+    return;
+  }
+
   request->state = DOWNLOAD_REQUEST_STATE_OPEN;
   request->download_request_time =
       gst_adaptive_demux_clock_get_time (dh->clock);
@@ -699,8 +922,13 @@ submit_transfer (DownloadHelper * dh, GTask * transfer_task)
   transfer_task_report_progress (transfer_task);
   download_request_unlock (request);
 
-  _soup_session_send_async (dh->session, transfer->msg, transfer->cancellable,
-      on_request_sent, transfer_task);
+  if (transfer->is_file_transfer) {
+    g_file_read_async (transfer->file, G_PRIORITY_DEFAULT,
+        transfer->cancellable, on_file_ready, transfer_task);
+  } else {
+    _soup_session_send_async (dh->session, transfer->msg, transfer->cancellable,
+        on_request_sent, transfer_task);
+  }
   g_array_append_val (dh->active_transfers, transfer_task);
 }
 
@@ -799,15 +1027,14 @@ downloadhelper_stop (DownloadHelper * dh)
     DownloadRequest *request = transfer->request;
 
     download_request_lock (request);
-    /* Reset the state to UNSENT, to indicate cancellation, like an XMLHttpRequest does */
-    request->state = DOWNLOAD_REQUEST_STATE_UNSENT;
+    request->state = DOWNLOAD_REQUEST_STATE_CANCELLED;
     download_request_unlock (request);
 
     transfer->complete = TRUE;
     if (transfer->blocking)
       g_cond_broadcast (&transfer->cond);
-    else
-      g_task_return_boolean (transfer_task, TRUE);
+
+    g_task_return_boolean (transfer_task, TRUE);
   }
 
   g_array_set_size (dh->active_transfers, 0);
@@ -820,14 +1047,8 @@ downloadhelper_submit_request (DownloadHelper * dh,
     GError ** err)
 {
   GTask *transfer_task = NULL;
-  const gchar *method;
-  SoupMessage *msg;
-  SoupMessageHeaders *msg_headers;
   gboolean blocking = (flags & DOWNLOAD_FLAG_BLOCKING) != 0;
 
-  method =
-      (flags & DOWNLOAD_FLAG_HEADERS_ONLY) ? SOUP_METHOD_HEAD : SOUP_METHOD_GET;
-
   download_request_lock (request);
   if (request->in_use) {
     GST_ERROR ("Request for URI %s reusing active request object",
@@ -839,65 +1060,84 @@ downloadhelper_submit_request (DownloadHelper * dh,
   /* Clear the state back to unsent */
   request->state = DOWNLOAD_REQUEST_STATE_UNSENT;
 
-  msg = _soup_message_new (method, request->uri);
-  if (msg == NULL) {
-    g_set_error (err, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
-        "Could not parse download URI %s", request->uri);
-
-    request->state = DOWNLOAD_REQUEST_STATE_ERROR;
+  if (g_str_has_prefix (request->uri, "file://")) {
+    if (flags & DOWNLOAD_FLAG_HEADERS_ONLY) {
+      /* FIXME: Implement fake header only requests for file URIs? */
+      GST_ERROR ("file:// URIs do not support header-only requests");
+      download_request_unlock (request);
+      return FALSE;
+    }
     download_request_unlock (request);
 
-    return FALSE;
-  }
+    /* If resubmitting a request, clear any stale / unused data */
+    download_request_begin_download (request);
 
-  /* NOTE: There was a bug where Akamai servers return the
-   * wrong result for a range request on small files. To avoid
-   * it if the range starts within the first KB of the file, just
-   * start at 0 instead */
-  if (request->range_start < 1024)
-    request->range_start = 0;
+    transfer_task = transfer_task_new_file (dh, request, blocking);
+    g_mutex_lock (&dh->transfer_lock);
+  } else {
+    const gchar *method =
+        (flags & DOWNLOAD_FLAG_HEADERS_ONLY) ? SOUP_METHOD_HEAD :
+        SOUP_METHOD_GET;
 
-  msg_headers = _soup_message_get_request_headers (msg);
+    SoupMessage *msg = _soup_message_new (method, request->uri);
+    if (msg == NULL) {
+      g_set_error (err, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
+          "Could not parse download URI %s", request->uri);
 
-  if (request->range_start != 0 || request->range_end != -1) {
-    _soup_message_headers_set_range (msg_headers, request->range_start,
-        request->range_end);
-  }
+      request->state = DOWNLOAD_REQUEST_STATE_ERROR;
+      download_request_unlock (request);
 
-  download_request_unlock (request);
-
-  /* If resubmitting a request, clear any stale / unused data */
-  download_request_begin_download (request);
-
-  if ((flags & DOWNLOAD_FLAG_COMPRESS) == 0) {
-    _soup_message_disable_feature (msg, _soup_content_decoder_get_type ());
-  }
-  if (flags & DOWNLOAD_FLAG_FORCE_REFRESH) {
-    _soup_message_headers_append (msg_headers, "Cache-Control", "max-age=0");
-  }
-
-  /* Take the lock to protect header strings */
-  g_mutex_lock (&dh->transfer_lock);
-
-  if (referer != NULL) {
-    _soup_message_headers_append (msg_headers, "Referer", referer);
-  } else if (dh->referer != NULL) {
-    _soup_message_headers_append (msg_headers, "Referer", dh->referer);
-  }
-
-  if (dh->user_agent != NULL) {
-    _soup_message_headers_append (msg_headers, "User-Agent", dh->user_agent);
-  }
-
-  if (dh->cookies != NULL) {
-    gchar **cookie;
-
-    for (cookie = dh->cookies; *cookie != NULL; cookie++) {
-      _soup_message_headers_append (msg_headers, "Cookie", *cookie);
+      return FALSE;
     }
-  }
 
-  transfer_task = transfer_task_new (dh, request, msg, blocking);
+    /* NOTE: There was a bug where Akamai servers return the
+     * wrong result for a range request on small files. To avoid
+     * it if the range starts within the first KB of the file, just
+     * start at 0 instead */
+    if (request->range_start < 1024)
+      request->range_start = 0;
+
+    SoupMessageHeaders *msg_headers = _soup_message_get_request_headers (msg);
+
+    if (request->range_start != 0 || request->range_end != -1) {
+      _soup_message_headers_set_range (msg_headers, request->range_start,
+          request->range_end);
+    }
+
+    download_request_unlock (request);
+
+    /* If resubmitting a request, clear any stale / unused data */
+    download_request_begin_download (request);
+
+    if ((flags & DOWNLOAD_FLAG_COMPRESS) == 0) {
+      _soup_message_disable_feature (msg, _soup_content_decoder_get_type ());
+    }
+    if (flags & DOWNLOAD_FLAG_FORCE_REFRESH) {
+      _soup_message_headers_append (msg_headers, "Cache-Control", "max-age=0");
+    }
+
+    /* Take the lock to protect header strings */
+    g_mutex_lock (&dh->transfer_lock);
+
+    if (referer != NULL) {
+      _soup_message_headers_append (msg_headers, "Referer", referer);
+    } else if (dh->referer != NULL) {
+      _soup_message_headers_append (msg_headers, "Referer", dh->referer);
+    }
+
+    if (dh->user_agent != NULL) {
+      _soup_message_headers_append (msg_headers, "User-Agent", dh->user_agent);
+    }
+
+    if (dh->cookies != NULL) {
+      _soup_cookies_to_request (dh->cookies, msg);
+    }
+
+    transfer_task = transfer_task_new_soup (dh, request, msg, blocking);
+
+    g_signal_connect (msg, "restarted", G_CALLBACK (soup_msg_restarted_cb),
+        transfer_task);
+  }
 
   if (!dh->running) {
     /* The download helper was deactivated just as we went to dispatch this request.
@@ -920,9 +1160,6 @@ downloadhelper_submit_request (DownloadHelper * dh,
   request->in_use = TRUE;
   download_request_unlock (request);
 
-  g_signal_connect (msg, "restarted", G_CALLBACK (soup_msg_restarted_cb),
-      transfer_task);
-
   /* Now send the request over to the main loop for actual submission */
   GST_LOG ("Submitting transfer task %p", transfer_task);
   g_async_queue_push (dh->transfer_requests, transfer_task);
@@ -966,8 +1203,7 @@ downloadhelper_cancel_request (DownloadHelper * dh, DownloadRequest * request)
   GST_DEBUG ("Cancelling request for URI %s range %" G_GINT64_FORMAT " %"
       G_GINT64_FORMAT, request->uri, request->range_start, request->range_end);
 
-  request->state = DOWNLOAD_REQUEST_STATE_UNSENT;
-
+  request->state = DOWNLOAD_REQUEST_STATE_CANCELLED;
   for (i = dh->active_transfers->len - 1; i >= 0; i--) {
     GTask *transfer_task = g_array_index (dh->active_transfers, GTask *, i);
     DownloadHelperTransfer *transfer = g_task_get_task_data (transfer_task);
