@@ -29,7 +29,7 @@
  *
  * ## Example launch line
  * |[
- * gst-launch-1.0 -v filesrc location=some.png ! decodebin ! imagefreeze ! autovideosink
+ * gst-launch-1.0 -v filesrc location=some.png ! decodebin ! videoconvert ! imagefreeze ! autovideosink
  * ]| This pipeline shows a still frame stream of a PNG file.
  *
  */
@@ -89,18 +89,24 @@ static gboolean gst_image_freeze_src_query (GstPad * pad, GstObject * parent,
 static GstStaticPadTemplate sink_pad_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw(ANY)"));
+    GST_STATIC_CAPS
+    ("video/x-raw(ANY); video/x-bayer(ANY); image/jpeg(ANY); image/png(ANY)"));
 
 static GstStaticPadTemplate src_pad_template =
-GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw(ANY)"));
+    GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
+    GST_STATIC_CAPS
+    ("video/x-raw(ANY); video/x-bayer(ANY); image/jpeg(ANY); image/png(ANY)"));
 
 GST_DEBUG_CATEGORY_STATIC (gst_image_freeze_debug);
 #define GST_CAT_DEFAULT gst_image_freeze_debug
 
 #define gst_image_freeze_parent_class parent_class
 G_DEFINE_TYPE (GstImageFreeze, gst_image_freeze, GST_TYPE_ELEMENT);
-
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (imagefreeze, "imagefreeze",
+    GST_RANK_NONE, GST_TYPE_IMAGE_FREEZE,
+    GST_DEBUG_CATEGORY_INIT (gst_image_freeze_debug, "imagefreeze", 0,
+        "imagefreeze element");
+    );
 
 static void
 gst_image_freeze_class_init (GstImageFreezeClass * klass)
@@ -228,7 +234,7 @@ gst_image_freeze_sink_setcaps (GstImageFreeze * self, GstCaps * caps)
   gboolean ret = FALSE;
   GstStructure *s;
   gint fps_n, fps_d;
-  GstCaps *othercaps, *intersection;
+  GstCaps *othercaps, *intersection, *pad_current_caps;
   guint i, n;
   GstPad *pad;
 
@@ -241,8 +247,12 @@ gst_image_freeze_sink_setcaps (GstImageFreeze * self, GstCaps * caps)
   if (self->negotiated_framerate) {
     gst_caps_set_simple (caps, "framerate", GST_TYPE_FRACTION, self->fps_n,
         self->fps_d, NULL);
-    GST_DEBUG_OBJECT (pad, "Setting caps %" GST_PTR_FORMAT, caps);
-    gst_pad_set_caps (self->srcpad, caps);
+    pad_current_caps = gst_pad_get_current_caps (self->srcpad);
+    if (pad_current_caps && !gst_caps_is_equal (caps, pad_current_caps)) {
+      GST_DEBUG_OBJECT (pad, "Setting caps %" GST_PTR_FORMAT, caps);
+      gst_pad_set_caps (self->srcpad, caps);
+    }
+    gst_caps_unref (pad_current_caps);
     gst_caps_unref (caps);
     return TRUE;
   }
@@ -688,7 +698,7 @@ gst_image_freeze_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
           &stop_type, &stop);
       gst_event_unref (event);
 
-      flush = ! !(flags & GST_SEEK_FLAG_FLUSH);
+      flush = !!(flags & GST_SEEK_FLAG_FLUSH);
 
       if (format != GST_FORMAT_TIME && format != GST_FORMAT_DEFAULT) {
         GST_ERROR_OBJECT (pad, "Seek in invalid format: %s",
@@ -1237,14 +1247,7 @@ gst_image_freeze_provide_clock (GstElement * element)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (gst_image_freeze_debug, "imagefreeze", 0,
-      "imagefreeze element");
-
-  if (!gst_element_register (plugin, "imagefreeze", GST_RANK_NONE,
-          GST_TYPE_IMAGE_FREEZE))
-    return FALSE;
-
-  return TRUE;
+  return GST_ELEMENT_REGISTER (imagefreeze, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
