@@ -347,6 +347,8 @@ struct _QtDemuxStreamStsdEntry
   guint bytes_per_sample;
   guint bytes_per_frame;
   guint compression;
+  gboolean needs_reorder;
+  gint reorder_map[64];
 
   /* if we use chunks or samples */
   gboolean sampled;
@@ -420,6 +422,8 @@ struct _QtDemuxStream
                                  * Currently only set for raw audio streams*/
 
   /* video info */
+  GstVideoInfo info;
+
   /* aspect ratio */
   gint display_width;
   gint display_height;
@@ -430,6 +434,7 @@ struct _QtDemuxStream
   GstAllocationParams params;
 
   gsize alignment;
+  gsize stride;
 
   /* when a discontinuity is pending */
   gboolean discont;
