@@ -162,6 +162,14 @@
 #define GST_MATROSKA_ID_LUMINANCEMAX               0x55D9
 #define GST_MATROSKA_ID_LUMINANCEMIN               0x55DA
 
+#define GST_MATROSKA_ID_VIDEOPROJECTION            0x7670
+/* IDs in the Projection master*/
+#define GST_MATROSKA_ID_VIDEOPROJECTIONTYPE        0x7671
+#define GST_MATROSKA_ID_VIDEOPROJECTIONPRIVATE     0x7672
+#define GST_MATROSKA_ID_VIDEOPROJECTIONPOSEYAW     0x7673
+#define GST_MATROSKA_ID_VIDEOPROJECTIONPOSEPITCH   0x7674
+#define GST_MATROSKA_ID_VIDEOPROJECTIONPOSEROLL    0x7675
+
 /* IDs in the TrackAudio master */
 #define GST_MATROSKA_ID_AUDIOSAMPLINGFREQ          0xB5
 #define GST_MATROSKA_ID_AUDIOBITDEPTH              0x6264
@@ -379,6 +387,7 @@
 #define GST_MATROSKA_CODEC_ID_VIDEO_VP9          "V_VP9"
 #define GST_MATROSKA_CODEC_ID_VIDEO_AV1          "V_AV1"
 #define GST_MATROSKA_CODEC_ID_VIDEO_MPEGH_HEVC   "V_MPEGH/ISO/HEVC"
+#define GST_MATROSKA_CODEC_ID_VIDEO_MPEGI_VVC    "V_MPEGI/ISO/VVC"
 #define GST_MATROSKA_CODEC_ID_VIDEO_PRORES       "V_PRORES"
 #define GST_MATROSKA_CODEC_ID_VIDEO_FFV1         "V_FFV1"
 
