@@ -26,6 +26,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/audio/audio.h>
 
+#include "gstrtpelements.h"
 #include "gstrtpceltdepay.h"
 #include "gstrtputils.h"
 
@@ -74,7 +75,8 @@ static gboolean gst_rtp_celt_depay_setcaps (GstRTPBaseDepayload * depayload,
 #define gst_rtp_celt_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpCELTDepay, gst_rtp_celt_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
-
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpceltdepay, "rtpceltdepay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_CELT_DEPAY, rtp_element_init (plugin));
 static void
 gst_rtp_celt_depay_class_init (GstRtpCELTDepayClass * klass)
 {
@@ -87,10 +89,10 @@ gst_rtp_celt_depay_class_init (GstRtpCELTDepayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstrtpbasedepayload_class = (GstRTPBaseDepayloadClass *) klass;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_celt_depay_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_celt_depay_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_celt_depay_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_celt_depay_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP CELT depayloader", "Codec/Depayloader/Network/RTP",
@@ -260,18 +262,10 @@ gst_rtp_celt_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
         GST_TIME_ARGS (GST_BUFFER_PTS (outbuf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
 
-    gst_rtp_drop_meta (GST_ELEMENT_CAST (depayload), outbuf,
-        g_quark_from_static_string (GST_META_TAG_AUDIO_STR));
+    gst_rtp_drop_non_audio_meta (depayload, outbuf);
 
     gst_rtp_base_depayload_push (depayload, outbuf);
   }
 
   return NULL;
 }
-
-gboolean
-gst_rtp_celt_depay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpceltdepay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_CELT_DEPAY);
-}
