@@ -25,6 +25,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include "gstrtpelements.h"
 #include "gstrtpg729depay.h"
 #include "gstrtputils.h"
 
@@ -80,6 +81,8 @@ static GstBuffer *gst_rtp_g729_depay_process (GstRTPBaseDepayload * depayload,
 #define gst_rtp_g729_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpG729Depay, gst_rtp_g729_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpg729depay, "rtpg729depay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_G729_DEPAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_g729_depay_class_init (GstRtpG729DepayClass * klass)
@@ -93,10 +96,10 @@ gst_rtp_g729_depay_class_init (GstRtpG729DepayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstrtpbasedepayload_class = (GstRTPBaseDepayloadClass *) klass;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_g729_depay_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_g729_depay_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_g729_depay_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_g729_depay_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP G.729 depayloader", "Codec/Depayloader/Network/RTP",
@@ -202,8 +205,7 @@ gst_rtp_g729_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_RESYNC);
   }
 
-  gst_rtp_drop_meta (GST_ELEMENT_CAST (depayload), outbuf,
-      g_quark_from_static_string (GST_META_TAG_AUDIO_STR));
+  gst_rtp_drop_non_audio_meta (depayload, outbuf);
 
   GST_LOG_OBJECT (depayload, "pushing buffer of size %" G_GSIZE_FORMAT,
       gst_buffer_get_size (outbuf));
@@ -217,10 +219,3 @@ bad_packet:
     return NULL;
   }
 }
-
-gboolean
-gst_rtp_g729_depay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpg729depay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_G729_DEPAY);
-}
