@@ -108,6 +108,8 @@ struct _RTPJitterBuffer {
   GstClock      *media_clock;
   gulong         media_clock_synced_id;
   guint64        media_clock_offset;
+  gint64         media_clock_correction;
+  gboolean       media_clock_reference_timestamp_meta_only;
 
   gboolean       rfc7273_sync;
 };
@@ -172,7 +174,7 @@ void                  rtp_jitter_buffer_set_delay        (RTPJitterBuffer *jbuf,
 void                  rtp_jitter_buffer_set_clock_rate   (RTPJitterBuffer *jbuf, guint32 clock_rate);
 guint32               rtp_jitter_buffer_get_clock_rate   (RTPJitterBuffer *jbuf);
 
-void                  rtp_jitter_buffer_set_media_clock  (RTPJitterBuffer *jbuf, GstClock * clock, guint64 clock_offset);
+void                  rtp_jitter_buffer_set_media_clock  (RTPJitterBuffer *jbuf, GstClock * clock, guint64 clock_offset, gint64 clock_correction, gboolean reference_timestamp_meta_only);
 void                  rtp_jitter_buffer_set_pipeline_clock (RTPJitterBuffer *jbuf, GstClock * clock);
 
 gboolean              rtp_jitter_buffer_get_rfc7273_sync (RTPJitterBuffer *jbuf);
@@ -210,7 +212,7 @@ void                  rtp_jitter_buffer_get_sync         (RTPJitterBuffer *jbuf,
 
 GstClockTime          rtp_jitter_buffer_calculate_pts    (RTPJitterBuffer * jbuf, GstClockTime dts, gboolean estimated_dts,
                                                           guint32 rtptime, GstClockTime base_time, gint gap,
-                                                          gboolean is_rtx);
+                                                          gboolean is_rtx, GstClockTime * p_ntp_time);
 
 gboolean              rtp_jitter_buffer_can_fast_start   (RTPJitterBuffer * jbuf, gint num_packet);
 
