@@ -31,8 +31,8 @@
 #include <gst/gst.h>
 
 #include "gstv4l2object.h"
-#include "v4l2_calls.h"
 #include "v4l2-utils.h"
+#include "gstv4l2elements.h"
 
 #ifdef HAVE_GUDEV
 #include <gudev/gudev.h>
@@ -45,11 +45,13 @@ static GstV4l2Device *gst_v4l2_device_new (const gchar * device_path,
 
 G_DEFINE_TYPE (GstV4l2DeviceProvider, gst_v4l2_device_provider,
     GST_TYPE_DEVICE_PROVIDER);
+GST_DEVICE_PROVIDER_REGISTER_DEFINE (v4l2deviceprovider, "v4l2deviceprovider",
+    GST_RANK_PRIMARY, GST_TYPE_V4L2_DEVICE_PROVIDER);
 
 static void gst_v4l2_device_provider_finalize (GObject * object);
 static GList *gst_v4l2_device_provider_probe (GstDeviceProvider * provider);
 
-#if HAVE_GUDEV
+#ifdef HAVE_GUDEV
 static gboolean gst_v4l2_device_provider_start (GstDeviceProvider * provider);
 static void gst_v4l2_device_provider_stop (GstDeviceProvider * provider);
 #endif
@@ -63,7 +65,7 @@ gst_v4l2_device_provider_class_init (GstV4l2DeviceProviderClass * klass)
 
   dm_class->probe = gst_v4l2_device_provider_probe;
 
-#if HAVE_GUDEV
+#ifdef HAVE_GUDEV
   dm_class->start = gst_v4l2_device_provider_start;
   dm_class->stop = gst_v4l2_device_provider_stop;
 #endif
@@ -79,7 +81,7 @@ gst_v4l2_device_provider_class_init (GstV4l2DeviceProviderClass * klass)
 static void
 gst_v4l2_device_provider_init (GstV4l2DeviceProvider * provider)
 {
-#if HAVE_GUDEV
+#ifdef HAVE_GUDEV
   g_cond_init (&provider->started_cond);
 #endif
 }
@@ -87,7 +89,7 @@ gst_v4l2_device_provider_init (GstV4l2DeviceProvider * provider)
 static void
 gst_v4l2_device_provider_finalize (GObject * object)
 {
-#if HAVE_GUDEV
+#ifdef HAVE_GUDEV
   GstV4l2DeviceProvider *provider = GST_V4L2_DEVICE_PROVIDER (object);
 
   g_cond_clear (&provider->started_cond);
@@ -114,10 +116,10 @@ gst_v4l2_device_provider_probe_device (GstV4l2DeviceProvider * provider,
   if (!S_ISCHR (st.st_mode))
     goto destroy;
 
-  v4l2obj = gst_v4l2_object_new ((GstElement *) provider,
+  v4l2obj = gst_v4l2_object_new (NULL, GST_OBJECT (provider),
       V4L2_BUF_TYPE_VIDEO_CAPTURE, device_path, NULL, NULL, NULL);
 
-  if (!gst_v4l2_open (v4l2obj))
+  if (!gst_v4l2_open (v4l2obj, NULL))
     goto destroy;
 
   gst_structure_set (props, "device.api", G_TYPE_STRING, "v4l2", NULL);
@@ -136,22 +138,36 @@ gst_v4l2_device_provider_probe_device (GstV4l2DeviceProvider * provider,
   gst_structure_set (props, "v4l2.device.device_caps", G_TYPE_UINT,
       v4l2obj->vcap.device_caps, NULL);
 
-  if (v4l2obj->vcap.capabilities & V4L2_CAP_VIDEO_CAPTURE)
-    type = GST_V4L2_DEVICE_TYPE_SOURCE;
-
-  if (v4l2obj->vcap.capabilities & V4L2_CAP_VIDEO_OUTPUT) {
-    /* Morph it in case our initial guess was wrong */
-    v4l2obj->type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
-
-    if (type == GST_V4L2_DEVICE_TYPE_INVALID)
-      type = GST_V4L2_DEVICE_TYPE_SINK;
-    else
-      /* We ignore M2M devices that are both capture and output for now
-       * The provider is not for them
-       */
+  if (v4l2obj->device_caps &
+      (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VIDEO_CAPTURE_MPLANE)) {
+    /* We ignore touch sensing devices; those are't really video */
+    if (v4l2obj->device_caps & V4L2_CAP_TOUCH)
       goto close;
+
+    type = GST_V4L2_DEVICE_TYPE_SOURCE;
+    v4l2obj->skip_try_fmt_probes = TRUE;
   }
 
+  if (v4l2obj->device_caps &
+      (V4L2_CAP_VIDEO_OUTPUT | V4L2_CAP_VIDEO_OUTPUT_MPLANE)) {
+    /* We ignore M2M devices that are both capture and output for now
+     * The provider is not for them */
+    if (type != GST_V4L2_DEVICE_TYPE_INVALID)
+      goto close;
+
+    type = GST_V4L2_DEVICE_TYPE_SINK;
+
+    /* We have opened as a capture as we didn't know, now that know,
+     * let's fixed it */
+    if (v4l2obj->device_caps & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
+      v4l2obj->type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
+    else
+      v4l2obj->type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
+  }
+
+  if (type == GST_V4L2_DEVICE_TYPE_INVALID)
+    goto close;
+
   caps = gst_v4l2_object_get_caps (v4l2obj, NULL);
 
   if (caps == NULL)
@@ -211,7 +227,7 @@ gst_v4l2_device_provider_probe (GstDeviceProvider * provider)
   return devices;
 }
 
-#if HAVE_GUDEV
+#ifdef HAVE_GUDEV
 
 static GstDevice *
 gst_v4l2_device_provider_device_from_udev (GstV4l2DeviceProvider * provider,
@@ -353,6 +369,8 @@ provider_thread (gpointer data)
   if (context == NULL || loop == NULL) {
     provider->started = TRUE;
     g_cond_broadcast (&provider->started_cond);
+    g_clear_pointer (&loop, g_main_loop_unref);
+    g_clear_pointer (&context, g_main_context_unref);
     GST_OBJECT_UNLOCK (provider);
     return NULL;
   }
@@ -409,7 +427,7 @@ gst_v4l2_device_provider_start (GstDeviceProvider * provider)
   self->context = g_main_context_new ();
   self->loop = g_main_loop_new (self->context, FALSE);
 
-  self->thread = g_thread_new ("v4l2-device-provider", provider_thread,
+  self->thread = g_thread_new ("v4l2-dev-prvdr", provider_thread,
       g_object_ref (self));
 
   while (self->started == FALSE)
@@ -435,19 +453,24 @@ gst_v4l2_device_provider_stop (GstDeviceProvider * provider)
   self->loop = NULL;
   GST_OBJECT_UNLOCK (self);
 
-  if (!context || !loop)
+  if (!context || !loop) {
+    g_clear_pointer (&self->loop, g_main_loop_unref);
+    g_clear_pointer (&self->context, g_main_context_unref);
     return;
+  }
 
   idle_stop_source = g_idle_source_new ();
   g_source_set_callback (idle_stop_source, (GSourceFunc) g_main_loop_quit, loop,
-      (GDestroyNotify) g_main_loop_unref);
+      NULL);
   g_source_attach (idle_stop_source, context);
   g_source_unref (idle_stop_source);
-  g_main_context_unref (context);
 
   g_thread_join (self->thread);
   self->thread = NULL;
   self->started = FALSE;
+
+  g_main_loop_unref (loop);
+  g_main_context_unref (context);
 }
 
 #endif
