@@ -22,11 +22,18 @@
  * SECTION:element-ximagesrc
  * @title: ximagesrc
  *
- * This element captures your X Display and creates raw RGB video.  It uses
+ * This element captures your X Display and creates raw RGB video. It uses
  * the XDamage extension if available to only capture areas of the screen that
- * have changed since the last frame.  It uses the XFixes extension if
- * available to also capture your mouse pointer.  By default it will fixate to
- * 25 frames per second.
+ * have changed since the last frame. It uses the XFixes extension if
+ * available to also capture your mouse pointer. It supports handling of
+ * mouse and keyboard events. By default it will fixate to 25 frames per second.
+ * 
+ * Applications are expected to call `XinitThreads()` before any other threads
+ * are started. For use in gst-launch-1.0 or other GStreamer command line 
+ * applications it is also possible to set the GST_XINITTHREADS=1 environment 
+ * variable so that `XInitThreads()` gets called when the plugin is loaded. This 
+ * may be too late in other use case scenarios though, so applications should 
+ * not rely on that.
  *
  * ## Example pipelines
  * |[
@@ -39,6 +46,7 @@
 #include "config.h"
 #endif
 #include "gstximagesrc.h"
+#include "gst-ximage-navigation.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -47,7 +55,7 @@
 #include <X11/Xutil.h>
 
 #include <gst/gst.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 #include <gst/video/video.h>
 
 #include "gst/glib-compat-private.h"
@@ -55,6 +63,14 @@
 GST_DEBUG_CATEGORY_STATIC (gst_debug_ximage_src);
 #define GST_CAT_DEFAULT gst_debug_ximage_src
 
+#define MOUSE_SCROLL_UP_BUTTON 4
+#define MOUSE_SCROLL_DOWN_BUTTON 5
+
+#define KEYCODE_CTRL 0x25
+#define KEYCODE_SHIFT 0x32
+#define KEYCODE_ALT 0x40
+#define KEYCODE_META 0x85
+
 static GstStaticPadTemplate t =
 GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw, "
@@ -75,6 +91,7 @@ enum
   PROP_REMOTE,
   PROP_XID,
   PROP_XNAME,
+  PROP_ENABLE_NAVIGATION_EVENTS
 };
 
 #define gst_ximage_src_parent_class parent_class
@@ -364,12 +381,23 @@ gst_ximage_src_unlock (GstBaseSrc * basesrc)
 static gboolean
 gst_ximage_src_recalc (GstXImageSrc * src)
 {
-  if (!src->xcontext)
-    return FALSE;
+  XWindowAttributes attrs;
+  gboolean has_changed = FALSE;
 
-  /* Maybe later we can check the display hasn't changed size */
-  /* We could use XQueryPointer to get only the current window. */
-  return TRUE;
+  g_assert (src->xwindow != 0 && src->xcontext);
+
+  if (XGetWindowAttributes (src->xcontext->disp, src->xwindow, &attrs)) {
+    if (src->xwin_width != attrs.width) {
+      src->xwin_width = attrs.width;
+      has_changed = TRUE;
+    }
+    if (src->xwin_height != attrs.height) {
+      src->xwin_height = attrs.height;
+      has_changed = TRUE;
+    }
+  }
+
+  return has_changed;
 }
 
 #ifdef HAVE_XFIXES
@@ -500,7 +528,7 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     ximagesrc->buffer_pool = g_slist_delete_link (ximagesrc->buffer_pool,
         ximagesrc->buffer_pool);
 
-    if ((meta->width == ximagesrc->width) ||
+    if ((meta->width == ximagesrc->width) &&
         (meta->height == ximagesrc->height))
       break;
 
@@ -547,9 +575,8 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     do {
       XDamageNotifyEvent *damage_ev = (XDamageNotifyEvent *) (&ev);
 
-      XNextEvent (ximagesrc->xcontext->disp, &ev);
-
-      if (ev.type == ximagesrc->damage_event_base + XDamageNotify &&
+      if (XCheckTypedEvent (ximagesrc->xcontext->disp,
+              ximagesrc->damage_event_base + XDamageNotify, &ev) &&
           damage_ev->level == XDamageReportNonEmpty) {
 
         XDamageSubtract (ximagesrc->xcontext->disp, ximagesrc->damage, None,
@@ -833,12 +860,6 @@ gst_ximage_src_create (GstPushSrc * bs, GstBuffer ** buf)
   GstClockTime dur;
   gint64 next_frame_no;
 
-  if (!gst_ximage_src_recalc (s)) {
-    GST_ELEMENT_ERROR (s, RESOURCE, FAILED,
-        (_("Changing resolution at runtime is not yet supported.")), (NULL));
-    return GST_FLOW_ERROR;
-  }
-
   if (s->fps_n <= 0 || s->fps_d <= 0)
     return GST_FLOW_NOT_NEGOTIATED;     /* FPS must be > 0 */
 
@@ -907,7 +928,19 @@ gst_ximage_src_create (GstPushSrc * bs, GstBuffer ** buf)
   s->last_frame_no = next_frame_no;
   GST_OBJECT_UNLOCK (s);
 
+  XGrabServer (s->xcontext->disp);
+
+  if (gst_ximage_src_recalc (s) && !gst_base_src_negotiate (GST_BASE_SRC (s))) {
+    XUngrabServer (s->xcontext->disp);
+    XSync (s->xcontext->disp, False);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+
   image = gst_ximage_src_ximage_get (s);
+
+  XUngrabServer (s->xcontext->disp);
+  XSync (s->xcontext->disp, False);
+
   if (!image)
     return GST_FLOW_ERROR;
 
@@ -927,7 +960,6 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_DISPLAY_NAME:
-
       g_free (src->display_name);
       src->display_name = g_value_dup_string (value);
       break;
@@ -945,9 +977,13 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
       break;
     case PROP_ENDX:
       src->endx = g_value_get_uint (value);
+      /* property comments say 0 means right/bottom, means we can't capture
+       * the top left pixel alone */
+      src->endx_fit_to_screen = (src->endx == 0);
       break;
     case PROP_ENDY:
       src->endy = g_value_get_uint (value);
+      src->endy_fit_to_screen = (src->endy == 0);
       break;
     case PROP_REMOTE:
       src->remote = g_value_get_boolean (value);
@@ -967,6 +1003,9 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
       g_free (src->xname);
       src->xname = g_value_dup_string (value);
       break;
+    case PROP_ENABLE_NAVIGATION_EVENTS:
+      src->enable_navigation_events = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -984,7 +1023,6 @@ gst_ximage_src_get_property (GObject * object, guint prop_id,
         g_value_set_string (value, DisplayString (src->xcontext->disp));
       else
         g_value_set_string (value, src->display_name);
-
       break;
     case PROP_SHOW_POINTER:
       g_value_set_boolean (value, src->show_pointer);
@@ -1013,6 +1051,9 @@ gst_ximage_src_get_property (GObject * object, guint prop_id,
     case PROP_XNAME:
       g_value_set_string (value, src->xname);
       break;
+    case PROP_ENABLE_NAVIGATION_EVENTS:
+      g_value_set_boolean (value, src->enable_navigation_events);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1070,32 +1111,21 @@ gst_ximage_src_get_caps (GstBaseSrc * bs, GstCaps * filter)
   if ((!s->xcontext) && (!gst_ximage_src_open_display (s, s->display_name)))
     return gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->srcpad);
 
-  if (!gst_ximage_src_recalc (s))
-    return gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->srcpad);
+  gst_ximage_src_recalc (s);
 
   xcontext = s->xcontext;
-  width = s->xcontext->width;
-  height = s->xcontext->height;
-  if (s->xwindow != 0) {
-    XWindowAttributes attrs;
-    int status = XGetWindowAttributes (s->xcontext->disp, s->xwindow, &attrs);
-    if (status) {
-      width = attrs.width;
-      height = attrs.height;
-    }
-  }
+  width = s->xwin_width;
+  height = s->xwin_height;
 
-  /* property comments say 0 means right/bottom, means we can't capture
-     the top left pixel alone */
-  if (s->endx == 0)
+  if (s->endx_fit_to_screen)
     s->endx = width - 1;
-  if (s->endy == 0)
+  if (s->endy_fit_to_screen)
     s->endy = height - 1;
 
   if (s->endx >= s->startx && s->endy >= s->starty) {
     /* this means user has put in values */
-    if (s->startx < xcontext->width && s->endx < xcontext->width &&
-        s->starty < xcontext->height && s->endy < xcontext->height) {
+    if (s->startx < width && s->endx < width &&
+        s->starty < height && s->endy < height) {
       /* values are fine */
       s->width = width = s->endx - s->startx + 1;
       s->height = height = s->endy - s->starty + 1;
@@ -1106,6 +1136,8 @@ gst_ximage_src_get_caps (GstBaseSrc * bs, GstCaps * filter)
       s->starty = 0;
       s->endx = width - 1;
       s->endy = height - 1;
+      s->endx_fit_to_screen = FALSE;
+      s->endy_fit_to_screen = FALSE;
     }
   } else {
     GST_WARNING ("User put in bogus co-ordinates, setting to full screen");
@@ -1113,6 +1145,8 @@ gst_ximage_src_get_caps (GstBaseSrc * bs, GstCaps * filter)
     s->starty = 0;
     s->endx = width - 1;
     s->endy = height - 1;
+    s->endx_fit_to_screen = FALSE;
+    s->endy_fit_to_screen = FALSE;
   }
   GST_DEBUG ("width = %d, height=%d", width, height);
 
@@ -1182,6 +1216,72 @@ gst_ximage_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
   return caps;
 }
 
+static gboolean
+gst_ximage_src_event (GstBaseSrc * base_src, GstEvent * event)
+{
+  gboolean ret = FALSE;
+#ifdef HAVE_XNAVIGATION
+  gboolean is_press = FALSE;
+  GstXImageSrc *src = GST_XIMAGE_SRC (base_src);
+  if (src->enable_navigation_events
+      && GST_EVENT_TYPE (event) == GST_EVENT_NAVIGATION) {
+    const gchar *key;
+    gint button;
+    gdouble x, y, delta_x, delta_y;
+
+    GST_DEBUG_OBJECT (src, "Processing event %" GST_PTR_FORMAT, event);
+    switch (gst_navigation_event_get_type (event)) {
+      case GST_NAVIGATION_EVENT_KEY_PRESS:
+        is_press = TRUE;        /* FALLTHROUGH */
+      case GST_NAVIGATION_EVENT_KEY_RELEASE:
+        if (gst_navigation_event_parse_key_event (event, &key)) {
+          gst_ximage_navigation_key (src->xcontext->disp, key, is_press);
+          ret = TRUE;
+        }
+        break;
+      case GST_NAVIGATION_EVENT_MOUSE_BUTTON_PRESS:
+        is_press = TRUE;        /* FALLTHROUGH */
+      case GST_NAVIGATION_EVENT_MOUSE_BUTTON_RELEASE:
+        if (gst_navigation_event_parse_mouse_button_event (event, &button, &x,
+                &y)) {
+          gst_ximage_navigation_mouse_push_button (src->xcontext->disp, button,
+              is_press);
+          ret = TRUE;
+        }
+        break;
+      case GST_NAVIGATION_EVENT_MOUSE_MOVE:
+        if (gst_navigation_event_parse_mouse_move_event (event, &x, &y)) {
+          gst_ximage_navigation_mouse_move_pointer (src->xcontext->disp,
+              (int) x, (int) y);
+          ret = TRUE;
+        }
+        break;
+      case GST_NAVIGATION_EVENT_MOUSE_SCROLL:
+        if (gst_navigation_event_parse_mouse_scroll_event (event, &x, &y,
+                &delta_x, &delta_y)) {
+          int scroll_button =
+              (int) delta_y <
+              0 ? MOUSE_SCROLL_DOWN_BUTTON : MOUSE_SCROLL_UP_BUTTON;
+          gst_ximage_navigation_mouse_push_button (src->xcontext->disp,
+              scroll_button, TRUE);
+          gst_ximage_navigation_mouse_push_button (src->xcontext->disp,
+              scroll_button, FALSE);
+          ret = TRUE;
+        }
+        break;
+      default:
+        break;
+    }
+  }
+#endif
+  if (!ret) {
+    ret =
+        GST_CALL_PARENT_WITH_DEFAULT (GST_BASE_SRC_CLASS, event, (base_src,
+            event), FALSE);
+  }
+  return ret;
+}
+
 static void
 gst_ximage_src_class_init (GstXImageSrcClass * klass)
 {
@@ -1281,6 +1381,17 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
           "Window name to capture from", NULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstXImageSrc:enable-navigation-events:
+   *
+   * Enable navigation events
+   */
+  g_object_class_install_property (gc, PROP_ENABLE_NAVIGATION_EVENTS,
+      g_param_spec_boolean ("enable-navigation-events",
+          "Enable navigation events",
+          "When enabled, navigation events are handled", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_element_class_set_static_metadata (ec, "Ximage video source",
       "Source/Video",
       "Creates a screenshot video stream",
@@ -1296,6 +1407,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   bc->stop = gst_ximage_src_stop;
   bc->unlock = gst_ximage_src_unlock;
   push_class->create = gst_ximage_src_create;
+  bc->event = gst_ximage_src_event;
 }
 
 static void
@@ -1312,7 +1424,10 @@ gst_ximage_src_init (GstXImageSrc * ximagesrc)
   ximagesrc->starty = 0;
   ximagesrc->endx = 0;
   ximagesrc->endy = 0;
+  ximagesrc->endx_fit_to_screen = TRUE;
+  ximagesrc->endy_fit_to_screen = TRUE;
   ximagesrc->remote = FALSE;
+  ximagesrc->enable_navigation_events = FALSE;
 }
 
 static gboolean
@@ -1320,6 +1435,9 @@ plugin_init (GstPlugin * plugin)
 {
   gboolean ret;
 
+  if (g_getenv ("GST_XINITTHREADS"))
+    XInitThreads ();
+
   GST_DEBUG_CATEGORY_INIT (gst_debug_ximage_src, "ximagesrc", 0,
       "ximagesrc element debug");
 
