@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <gst/check/gstcheck.h>
@@ -56,7 +56,7 @@ read_tags_from_file (const gchar * file, gboolean push_mode)
 
   /* kids, don't use a sync handler for this at home, really; we do because
    * we just want to abort and nothing else */
-  gst_bus_set_sync_handler (bus, error_cb, (gpointer) file);
+  gst_bus_set_sync_handler (bus, error_cb, (gpointer) file, NULL);
 
   src = gst_element_factory_make ("filesrc", "filesrc");
   fail_unless (src != NULL, "Failed to create 'filesrc' element!");
@@ -124,7 +124,7 @@ run_check_for_file (const gchar * filename, CheckTagsFunc * check_func)
   tags = read_tags_from_file (filename, FALSE);
   fail_unless (tags != NULL, "Failed to extract tags from '%s'", filename);
   check_func (tags, filename);
-  gst_tag_list_free (tags);
+  gst_tag_list_unref (tags);
 
   /* FIXME: need to fix id3demux for short content in push mode */
 #if 0
@@ -132,21 +132,22 @@ run_check_for_file (const gchar * filename, CheckTagsFunc * check_func)
   tags = read_tags_from_file (filename, TRUE);
   fail_unless (tags != NULL, "Failed to extract tags from '%s'", filename);
   check_func (tags, filename);
-  gst_tag_list_free (tags);
+  gst_tag_list_unref (tags);
 #endif
 }
 
 static void
 check_date_1977_06_23 (const GstTagList * tags, const gchar * file)
 {
-  GDate *date = NULL;
+  GstDateTime *date = NULL;
 
-  gst_tag_list_get_date (tags, GST_TAG_DATE, &date);
-  fail_unless (date != NULL, "Tags from %s should contain a GST_TAG_DATE tag");
-  fail_unless_equals_int (g_date_get_year (date), 1977);
-  fail_unless_equals_int (g_date_get_month (date), 6);
-  fail_unless_equals_int (g_date_get_day (date), 23);
-  g_date_free (date);
+  gst_tag_list_get_date_time (tags, GST_TAG_DATE_TIME, &date);
+  fail_unless (date != NULL,
+      "Tags from %s should contain a GST_TAG_DATE_TIME tag");
+  fail_unless_equals_int (gst_date_time_get_year (date), 1977);
+  fail_unless_equals_int (gst_date_time_get_month (date), 6);
+  fail_unless_equals_int (gst_date_time_get_day (date), 23);
+  gst_date_time_unref (date);
 }
 
 GST_START_TEST (test_tdat_tyer)
